extends Node2D

const FREQ_MAX = 11050.0 # 最大周波数
const VU_COUNT = 16 # 周波数の分解度
const MIN_DB = 60

var spectrum = null # スペクトラムエフェクト
var db_now = 0.0 # 現在の音量
var past_time = 0.0 # 経過時間
var db_list = [] # 音量の平均を取るためのリスト

onready var player = $AudioStreamPlayer2D
onready var label = $Label
onready var sprite = $AnimatedSprite

func _ready():
	# スペクトラムエフェクトを取得
	# バス番号1 (Voice)
	# エフェクト番号0 (Spectrum)
	spectrum = AudioServer.get_bus_effect_instance(1, 0)
	
func _process(_delta):
	if Input.is_action_just_pressed("ui_accept"):
		# ボイス再生
		player.stream = load("res://test1.mp3")
		#player.stream = load("res://test2.mp3")
		player.play()
	
	# 最大音量を取得.
	var max_energy = _get_max_energy()
	db_list.append(max_energy)
	
	# 経過時間を加算
	past_time += _delta
	if past_time > 0.2:
		# 0.2秒ごとに平均を求める
		_calc_db_average()
		past_time = 0
	
	if player.playing == false:
		# 再生していなければ音量を0にする
		db_now = 0
	
	# リップシンク更新
	_update_lipsync()

	label.text = "Volume: %3.2f"%db_now

func _calc_db_average():
	# 音量の平均を求める
	var sum = 0
	for v in db_list:
		sum += v
	# 平均を求める
	db_now = sum / db_list.size()
	db_list.clear()

func _get_max_energy():
	# 音量の最大値を取得する
	var prev_hz = 0
	var max_energy = 0
	
	# 各周波数ごとの音量から最大値を取り出す
	for i in range(1, VU_COUNT+1):
		var hz = i * FREQ_MAX / VU_COUNT;
		var magnitude: float = spectrum.get_magnitude_for_frequency_range(prev_hz, hz).length()
		var energy = clamp((MIN_DB + linear2db(magnitude)) / MIN_DB, 0, 1)
		if energy > max_energy:
			max_energy = energy # 最大音量
		prev_hz = hz
	
	return max_energy

func _update_lipsync():
	if db_now < 0.1:
		# 口パク停止
		sprite.play("default")
	else:
		if db_now > 0.4:
			# 大きい口パク
			sprite.play("lipsync2")
		else:
			# 小さい口パク
			sprite.play("lipsync1")
